import socket

from http.server import SimpleHTTPRequestHandler, ThreadingHTTPServer
from threading import Thread


class CustomDirHTTPRequestHandlerFactory:
    def __init__(self, output_dir):
        self.output_dir = output_dir

    def __call__(self, *args, **kwargs):
        return SimpleHTTPRequestHandler(*args, **kwargs, directory=self.output_dir)

class Server:
    def __init__(self, output_dir, port):
        handler = CustomDirHTTPRequestHandlerFactory(output_dir)
        srv_class = ThreadingHTTPServer

        infos = socket.getaddrinfo(
            None,
            port,
            type=socket.SOCK_STREAM,
            flags=socket.AI_PASSIVE,
        )
        srv_class.address_family, _, _, _, addr = next(iter(infos))

        handler.protocol = "HTTP/1.0"

        self.httpd = srv_class(addr, handler)
        self.host, self.port = self.httpd.socket.getsockname()[:2]

    def start(self):
        self.thread = Thread(target=self.run)
        self.thread.start()
        print("HTTP server running:", self.port)

    def shutdown(self):
        self.httpd.shutdown()

    def run(self):
        self.httpd.serve_forever()



import sys

from threading import Thread

from .nabu import NabuConfig, Nabu
from .watch import Watcher
from .serve import Server


def main(mode='render'):
    config = NabuConfig()
    n = Nabu(config)
    n.render_to_file()

    if mode == 'serve':
        server = Server(config['output_folder'], 8080)
        watcher = Watcher(config['content_folder'], n)

        server.start()
        watcher.start()

        try:
            while watcher.thread.is_alive():
                server.thread.join(0.1)
                watcher.thread.join(0.1)

        except KeyboardInterrupt:
            print('Stopping...')
            server.shutdown()
            watcher.shutdown()

            server.thread.join()
            watcher.thread.join()
        
    return n

if __name__ == "__main__":
    kwargs = {}

    if len(sys.argv) > 1:
        if sys.argv[1] == 'serve':
            kwargs.update(mode='serve')

    o = main(**kwargs)



import asyncio

from watchgod import DefaultDirWatcher, watch
from threading import Thread


class ContentWatcher(DefaultDirWatcher):
    def should_watch_file(self, entry):
        return entry.name.endswith('.md')

class Watcher:
    def __init__(self, content_folder, nabu_obj, watcher_cls=ContentWatcher):
        self.content_folder = content_folder
        self.watcher_cls = watcher_cls
        self.nabu = nabu_obj
        self.stop_event = asyncio.Event()

    def start(self):
        self.thread = Thread(target=self.run)
        self.thread.start()
        print("Watcher started!")

    def shutdown(self):
        self.stop_event.set()

    def run(self):
        for changes in watch(self.content_folder, stop_event=self.stop_event, watcher_cls=self.watcher_cls):
            self.nabu.render_to_file()



from collections import namedtuple
import frontmatter
import logging
import os
import yaml
import shutil

from collections import UserDict
from jinja2 import Template
from markdown2 import markdown
from pathlib import Path

from .menu import Menu

logger = logging.getLogger(__name__)


class NabuConfig(UserDict):
    def __init__(self, filename="config.yml"):
        with open(filename, "r") as fp:
            data = yaml.load(fp, Loader=yaml.FullLoader)

        super().__init__(data)

class Nabu:
    def __init__(self, config):
        self.config = config
        self.output_path = Path(config['output_folder'])
        self.content_path = Path(config['content_folder'])
        self.menu = Menu(config)

    def render_to_file(self):
        try:
            shutil.rmtree(self.output_path)
        except FileNotFoundError:
            pass
        finally:
            os.makedirs(self.output_path)
            shutil.copyfile(Path('assets/styles.css'), self.output_path / 'styles.css')
            shutil.copyfile(Path('assets/book.css'), self.output_path / 'book.css')
            shutil.copytree(Path('assets/fonts/'), self.output_path / 'fonts')

        self.menu.render_all(self.output_path)


from collections import UserDict
from os import stat
from pathlib import Path
from markdown2 import markdown

from .page import Page


class Menu(UserDict):
    def __init__(self, config):
        self.template = Path(config.get('template', 'templates/bootstrap/page.html'))

        menu = config['menu']
        basedir = Path(config.get('content_folder', 'content'))

        self.data = dict()

        self.root = Page(basedir, menu=self)
        self.root.slug = ''

        for chapter, pages in menu.items():
            parent_path = basedir / chapter
            parent = Page(parent_path, menu=self)

            if not pages and parent.metadata.get('autolist'):
                pages = parent.find_siblings()

            self.data[parent] = list()
            for page in pages:
                if not page.endswith('.md'):
                    page = page + '.md'
                path = basedir / chapter / page
                print(path)
                p = Page(path, parent=parent, menu=self)
                print(p)
                
                if p:
                    self.data[parent].append(p)

    @staticmethod
    def make_markdown_item(page, level):
        return '  ' * level + f" * [{page.title}]({page.url})\n"

    def markdown(self, pages=None, level=0):
        md = ''
        if pages is None:
            pages = self.data

        if isinstance(pages, list):
            md += ''.join([self.make_markdown_item(page, level) for page in pages])
        elif isinstance(pages, dict):
            for page in pages.keys():
                md += self.make_markdown_item(page, level)
                for subpage in pages[page]:
                    if isinstance(subpage, dict):
                        md += self.markdown(pages['page'], level+1)
                    else:
                        md += self.make_markdown_item(subpage, level+1)
        return md

    def render_all(self, output_path):
        self.root.render(output_path, self.template)
        for bundle in self.data:
            bundle.render(output_path, self.template)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py-nabu", # Replace with your own username
    version="0.1.2",
    author="Tim Guicherd",
    author_email="tim@grut.org",
    description="A small and efficient Markdown to HTML publishing platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/timoguic/nabu",
    packages=setuptools.find_packages(),
    install_requires=[
        'python-frontmatter',
        'Jinja2',
        'markdown2',
        'PyYAML',
        'watchgod',
    ],
    entry_points={
        'console_scripts': ['nabu=nabu.__main__:main'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)


import frontmatter
import glob
import logging
import os

from jinja2 import Template
from markdown2 import markdown
from pathlib import Path, PurePosixPath

logger = logging.getLogger(__name__)

class Page:
    def __new__(cls, source, **kwargs):
        if not source.exists() or not os.access(source, os.R_OK):
            return None

        instance = object.__new__(cls)
        return instance

    def __init__(self, source, parent=None, menu=None):
        if source.is_dir():
            source = source / 'index.md'

        self.source = source
        self.parent = parent
        self.menu = menu

        self.children = list()
        if parent:
            self.parent.add_child(self)

        self.metadata = dict()
        self.markdown = None
        try:
            with open(source, 'r') as fp:
                data = frontmatter.load(fp)
                self.metadata = data.metadata
                self.markdown = data.content
        except FileNotFoundError:
            pass

        if not self.parent:
            self.slug = self.source.parent.name
        else:
            self.slug = self.metadata.get('slug', f'{self.source.stem}.html')
            if not self.slug.endswith('.html'):
                self.slug = self.slug + '.html'

    def add_child(self, child):
        self.children.append(child)

    def find_siblings(self):
        paths = [Path(p).name for p in glob.glob(f'{self.source.parent}/*.md')]
        return [p for p in paths if p != 'index.md']

    @property
    def title(self):
        return self.metadata.get('title', self.source.parent.name)

    @property
    def url(self):
        url = self.build_output_dir(PurePosixPath('/'))
        if self.parent: url = url / self.slug
        else: url = url / 'index.html'

        return url

    def build_output_dir(self, basedir):
        if self.parent:
            return self.parent.build_output_dir(basedir)
        else:
            return Path(basedir) / self.slug

    def build_output_path(self, basedir):
        path = self.build_output_dir(basedir)

        if not path.exists():
            os.makedirs(path)

        if self.parent: return path / self.slug
        else: return path / 'index.html'

    def render(self, output_path, template_file):
        if not self.markdown:
            self.markdown = self.menu.markdown(self.children)

        if self.metadata.get('toc'):
            self.markdown += "\n\n## Table of contents\n"
            self.markdown += self.menu.markdown(self.children)

        content = markdown(self.markdown, extras={
            'fenced-code-blocks': None,
            'codehilite': None,
            'tables': None,
            'html-classes': {'table': 'table'}
        })

        with open(template_file, 'r') as fp:
            template = Template(fp.read())
            
        data = template.render(content=content, menu=self.menu, metadata=self.metadata)

        full_output_path = self.build_output_path(output_path)

        with open(full_output_path, 'w') as fp:
            success = fp.write(data)

        print("RENDER: ", self.source)
        for page in self.children:
            page.render(output_path, template_file)

        return success

    def __str__(self):
        return f"<Page '{self.source}'>"

    __repr__ = __str__
